#include <string.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <unistd.h>

class TCPclient
{
 private:
 
 int socket_id; // identificatore del socket
 
 public:
    
 // costruttore
 // inizializza il socket creando una connessione con il server
 // identificato da indirizzo IP e numero di porta TCP specificati
 TCPclient(char ip_address[], unsigned short port_number)
 {
  unsigned long binary_address = inet_network(ip_address);
  struct sockaddr_in add; // struttura per indirizzo
 
  // costruzione struttura indirizzo
  memset(&add, 0, sizeof(add)); // azzeramento struttura
  add.sin_family = AF_INET; // dominio indirizzi IP
  add.sin_addr.s_addr = htonl(binary_address); // indirizzo IP server
  add.sin_port = htons(port_number); // numero di porta server
  // apertura socket TCP
  if ((socket_id = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP)) < 0)
    {
     socket_id = -1;
     return;
    }
  // connessione al socket del server
  if (connect(socket_id, (struct sockaddr*)&add, sizeof(add)) < 0)
    {
	 close(socket_id);
     socket_id = -1;
     return;
    }
  // impostazione socket non bloccante
  if (fcntl(socket_id, F_SETFL, O_NONBLOCK) < 0)
	{
	 close(socket_id);
     socket_id = -1;
     return;
    }
  return;
 }
 
 // distruttore
  ~TCPclient(void)
 {
  if (socket_id >= 0)
    close(socket_id);
  return;
 }
  
 // verifica connessione
 bool isConnected(void)
 {
  if (socket_id >= 0)
    return true;
  else
      return false;
 }
 
 // trasmette al server num caratteri contenuti nel buffer
 // restituisce -1 in caso di errore, altrimenti il numero di
 // caratteri trasmessi
 int sendData(char buffer[], int num)
 {
  int n;
  
  if (socket_id < 0)
    return -1;
 
  // trasmissione dati
  if ((n = send(socket_id, (void*)buffer, (size_t)num, 0)) < 0)
    return -1;
  return n;
 }

 // riceve una sequenza di caratteri nel buffer di dimensione size
 // restituisce -1 in caso di errore o di dati non disponibili,
 // il numero di caratteri ricevuti in caso di successo
 int receiveData(char* buffer, int size)
 {
  int n;
  
  if (socket_id < 0)
    return -1;
 
  // ricezione dati (non bloccante)
  if ((n = recv(socket_id, (void*)buffer, (size_t)size, 0)) <= 0)
    return -1;
  return n;
 }
};

