
#include <iostream>
#include <cstring>
#include <cstdio>

#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <unistd.h>

#include "Products.h"

using namespace std;

#define TCP_PORT 12345


Products *products;

// funzione per la gestione della comunicazione con il client
void client_service(int socket_id)
{
 char buffer[256]; // buffer per ricezione
 char command[256]; // comando ricevuto da client (codice prodotto)
 unsigned int index; // indice per costruzione stringa di comando
 int i, n;
 PRODUCT product; // variabile per ricerca prodotto
 index = 0;
 while (1)
	  {
	   if ( (n = recv(socket_id, (void*)buffer, sizeof(buffer), 0)) <= 0)
		 {		
		  // chiusura della connessione da parte del client
		  close(socket_id);
		  return;
		 }
	   else
		   {
		    // ricezione caratteri: costruzione del comando
		    for (i=0; i<n; i++)
			   if (buffer[i] == '\r' || buffer[i] == '\n')
				 {
				  // comando completato: inserimento del terminatore di stringa
				  command[index] = '\0';
				  if (strlen(command) > 0)
				    {
					 // codice per richiesta informazioni prodotto
					 cout << "...ricevuta richiesta per codice: " << command << endl;
					 // ricerca prodotto
					 if (products->findProduct(command, &product) > 0)
					   {
					    // prodotto trovato: costruzione risposta
					    sprintf(buffer, "%s,%s,%f\r\n", product.code, product.description, product.price);
					    // trasmissione stringa di risposta
					    send(socket_id, (void*)buffer, strlen(buffer), 0);
					    cout << "...risposta con prodotto: " << buffer << endl;
					   }
					 else
                         {
                          strcpy(buffer, "ERRORE\r\n");
                          send(socket_id, (void*)buffer, strlen(buffer), 0);
                          cout << "...nessun prodotto trovato!" << endl << endl;
                         }
				    }
				  index = 0;
				  break;
				 }
			   else
				   {
				    // copia del carattere nel comando
				    if (index >= sizeof(command))
					  index = 0;
                    command[index] = buffer[i];
					index++;
				   }
		   }
      }
}


int main(void)
{
 int request_socket_id; // socket richieste di connessione
 int communication_socket_id; // socket comunicazione con client
 struct sockaddr_in server_add; // struttura per indirizzo server
 struct sockaddr_in client_add; // struttura per indirizzo client
 unsigned int client_add_size;

 // caricamento prodotti da file
 products = new Products("products.csv");
 if (products->getProductNumber() == 0)
   {
    cout << "Errore caricamento prodotti!" << endl;
    delete products;
    return -1;
   }
 // apertura del socket
 if ( (request_socket_id = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP)) < 0)
   {
    cout << "Errore apertura socket!" << endl;
    delete products;
    return -1;
   }
 // costruzione della struttura per l’indirizzo del server
 memset(&server_add, 0, sizeof(server_add));
 server_add.sin_family = AF_INET;
 server_add.sin_addr.s_addr = 0; // indirizzo IP locale
 server_add.sin_port = htons(TCP_PORT); // numero di porta server
 // associazione del numero di porta al socket
 if (bind( request_socket_id, (struct sockaddr *)&server_add, sizeof(server_add)) < 0)
   {
    cout << "Errore associazione socket!" << endl;
    close(request_socket_id);
    delete products;
    return -1;
   }
 // impostazione del socket per ricevere le richieste di connessione
 if (listen(request_socket_id, 0) < 0)
   {
    cout << "Errore impostazione socket!" << endl;
    close(request_socket_id);
    delete products;
    return -1;
   }
 cout << "Servizio attivo (caricati " << products->getProductNumber() << " prodotti)…" << endl;
 while (1)
	  {
	   communication_socket_id = accept(request_socket_id, (struct sockaddr*)&client_add, &client_add_size);
	   if (communication_socket_id >= 0)
		 {
		  // connessione effettuata: creazione nuovo processo per la gestione della comunicazione con il processo client
		  if (fork() == 0)
			{
             cout << "...connesione nuova cassa." << endl;
			 client_service(communication_socket_id);
			 close (communication_socket_id);
			 return 0;
			}
		  close (communication_socket_id);
		 }
	  }
 close(request_socket_id);
 return 0;
}
