
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <unistd.h>


// funzione di calcolo: restituisce -1 in caso di errore, 0 altrimenti
int calc(char operation[], double op1, double op2, double* result)
{
 if (strcmp(operation, "ADD") == 0)
   {
    *result = op1 + op2;
	return 0;
   }
 if (strcmp(operation, "SUB") == 0)
   {
	*result = op1 - op2;
	return 0;
   }
 if (strcmp(operation, "MUL") == 0)
   {
	*result = op1 * op2;
	return 0;
   }
 if ((strcmp(operation, "DIV") == 0) && (op2 != 0.0))
   {
	*result = op1 / op2;
	return 0;
   }
 return -1;
}

// funzione di gestione della comunicazione con il client
void client_service(int socket_id)
{
 char *request, *op1, *op2; 
 double result;
 unsigned char buffer[1024];
 char command[64], answer[64];
 int index, error;
 int i, n;

 
 index = 0;
 while (1)
	  {
	   if ((n = recv(socket_id, (void *)buffer, sizeof(buffer), 0)) <= 0)
		 {
		  // chiusura della connessione da parte del client
		  printf("Chiusura connessione.\r\n");
		  break;
		 }
	  else
	      {
		   // ricezione caratteri
		   for (i=0; i<n; i++)
			  if (buffer[i] == '\r' || buffer[i] == '\n' || buffer[i] == '\0')
				{
				 // ricezione completata
				 command[index] = '\0';
				 if (strlen(command) > 0)
				   {
					error = 1;
					request = strtok(command, ",\r\n\0");
					if (request != NULL)
					  {
					   if (strlen(request) == 3) // ADD, SUB, DIV o MUL
						 {
						  op1 = strtok(NULL, ",\r\n\0");
						  if (op1 != NULL)
							{
							 op2 = strtok(NULL, ",\r\n\0");
							 if (op2 != NULL)
							   {
							    if (strlen(op1)>0 && strlen(op2)>0)
								  {
								   if (calc(request, atof(op1), atof(op2), &result) == 0)
								     {
									  // invio risposta al client
									  sprintf(answer, "%f\r\n", result);
									  send(socket_id, answer, strlen(answer), 0);
									  error = 0;
									 }
								   }
								}
						    }
					   }
					}
				  }
				if (error) // invio messaggio di errore al client
				  send(socket_id, "ERROR\r\n", 7, 0);
				index = 0;
				break;
			   }
			 else
			 	 {
				  // copia del carattere nel comando
				  if (index >= sizeof(command))
				  index = 0;
				  command[index] = buffer[i];
				  index++;
				 }
	      }
      }
}


int main(int argc, char* argv[])
{
 int request_socket_id; // socket richieste di connessione
 int communication_socket_id; // socket comunicazione con client
 struct sockaddr_in server_add; // struttura per indirizzo server
 struct sockaddr_in client_add; // struttura per indirizzo client
 size_t client_add_size;
 int port;


 if (argc != 2)
   {
    printf("Errore argomenti!\r\n");
    return -1;
   }
 port = atoi(argv[1]);
 // apertura del socket
 if ( (request_socket_id = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP)) < 0)
  {
   printf("Errore apertura socket!\r\n");
   return -1;
  }
 // costruzione della struttura per l’indirizzo del server
 memset(&server_add, 0, sizeof(server_add));
 server_add.sin_family = AF_INET;
 server_add.sin_addr.s_addr = 0; // indirizzo IP locale
 server_add.sin_port = htons(port); // numero di porta server
 // associazione del numero di porta al socket
 if (bind( request_socket_id, (struct sockaddr *)&server_add, sizeof(server_add)) < 0)
   {
    printf("Errore associazione socket!\r\n");
    close(request_socket_id);
    return -1;
   }
 // impostazione del socket per ricevere le richieste di connessione
 if (listen(request_socket_id, 0) < 0)
   {
    printf("Errore impostazione socket!\r\n");
    close(request_socket_id);
    return -1;
   }
 printf("Servizio attivo…\r\n");
 while (1)
	  {
	   communication_socket_id = accept(request_socket_id, (struct sockaddr*)&client_add, &client_add_size);
	   if (communication_socket_id >= 0)
		 {
		  // connessione effettuata: creazione nuovo processo per la gestione della comunicazione con il processo client
		  if (fork() == 0)
			{
			 client_service(communication_socket_id);
			 close (communication_socket_id);
			 return 0;
			}
		  close (communication_socket_id);
		 }
	  }
 close(request_socket_id);
 return 0;
}

