
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <unistd.h>

#include "shared.h"
#include "semaphore.h"
#include "phonebook.h"

struct PHONEBOOK* shared;
int mutex;

// carica le voci della rubrica dal file specificato
// restituisce -1 in caso di errore, il numero di voci altrimenti
int loadFromFile(char filename[])
{
 FILE* file;
 char line[1024];
 char *name, *phone;
 int n;
 
 file = fopen(filename, "r");
 if (file == NULL)
   return -1;
 semaphore_wait(mutex);
 shared->N_entry = 0;
 semaphore_signal(mutex);
 do {
     fgets(line, sizeof(line)-1, file);
     name = strtok(line, ",\r\n\0");
     if (name != NULL)
      {
	   if (strlen(name) > 0)
         {
          phone = strtok(NULL, ",\r\n\0");
          if (phone != NULL)
            if (strlen(phone) > 0)
              {
			   semaphore_wait(mutex);
			   strncpy(shared->phonebook[shared->N_entry].name, name, 31);
			   strncpy(shared->phonebook[shared->N_entry].phone, phone, 15);
			   shared->N_entry++;
			   semaphore_signal(mutex);
			  }
		}
	  }
	} while (!feof(file));
 fclose(file);
 semaphore_wait(mutex);
 n = shared->N_entry;
 semaphore_signal(mutex);
 return n;
}

// salva le voci della rubrica nel file specificato
// restituisce -1 in caso di errore, il numero di voci altrimenti
int saveToFile(char filename[])
{
 FILE* file;
 int i, n;
 file = fopen(filename, "w");
 if (file == NULL)
   return -1;
 semaphore_wait(mutex);
 n = shared->N_entry;
 semaphore_signal(mutex);
 for (i=0; i<n; i++)
   {
	semaphore_wait(mutex);
	fprintf(file, "%s,%s\r\n", shared->phonebook[i].name, shared->phonebook[i].phone);
	semaphore_signal(mutex);
   }
 fclose(file);
 return n;
}

// ricerca di un nominativo o di un numero nella rubrica
// fornisce il corrispondente numero o nominativo
// restituisce 0 in caso di ricerca fallita, 1 altrimenti
int find(char in[], char *out)
{
 int i, n;
 
 semaphore_wait(mutex);
 n = shared->N_entry;
 semaphore_signal(mutex);
 for (i=0; i<n; i++)
	{
	 semaphore_wait(mutex);
	 if (strcmp(in, shared->phonebook[i].name) == 0)
	   {
		strncpy(out, shared->phonebook[i].phone, 15);
		semaphore_signal(mutex);
		return 1;
	   }
     if (strcmp(in, shared->phonebook[i].phone) == 0)
	   {
	    strncpy(out, shared->phonebook[i].name, 31);
	    semaphore_signal(mutex);
	    return 1;
	   }
     semaphore_signal(mutex);
    }
 return 0;
}

// aggiornamento del nominativo o numero di una voce della rubrica
// aggiunge una voce alla rubrica se non esistono nominativo/numero
// restituisce 1 in caso di aggiornamento, 2 in caso di aggiunta
int update(char name[], char phone[])
{
 int i, n;

 semaphore_wait(mutex);
 n = shared->N_entry;
 semaphore_signal(mutex);
 for (i=0; i<n; i++)
    {
     semaphore_wait(mutex);
     if (strcmp(name, shared->phonebook[i].name) == 0)
	   {
	    strncpy(shared->phonebook[i].phone, phone, 15);
	    semaphore_signal(mutex);
	    return 1;
	   }
	 if (strcmp(phone, shared->phonebook[i].phone) == 0)
	   {
	    strncpy(shared->phonebook[i].name, name, 31);
	    semaphore_signal(mutex);
	    return 1;
	   }
    semaphore_signal(mutex);
   }
 semaphore_wait(mutex);
 strncpy(shared->phonebook[i].name, name, 31);
 strncpy(shared->phonebook[i].phone, phone, 15);
 shared->N_entry++;
 semaphore_signal(mutex);
 return 2;
}




void client_service(int socket_id)
{
 char *request, *name, *phone, *name_or_phone;
 unsigned char buffer[64];
 char command[64], answer[64];
 int index, error;
 int i, n;
 index = 0;
 
 while (1)
	  {
	   if ((n = recv(socket_id, (void *)buffer, sizeof(buffer), 0))<= 0)
		 {
		  // chiusura della connessione da parte del client
		  printf("Chiusura connessione.\r\n");
		  return;
		 }
	   else
		   {
			// ricezione caratteri
			for (i=0; i<n; i++)
			   if ( buffer[i] == '\r' || buffer[i] == '\n' || buffer[i] == '\0')
				 {
				  // ricezione completata
				  command[index] = '\0';
				  if (strlen(command) > 0)
					{
					 error = 1;
					 request = strtok(command, ",\r\n\0");
					 if (request != NULL)
					   {
						if (strcmp(request, "GET") == 0)
						  {
						   // commando GET
						   name_or_phone = strtok(NULL, ",\r\n\0");
						   if (name_or_phone != NULL)
							 {
							  if (find(name_or_phone, answer) > 0)
								{
								 // invio risposta al client
								 strcat(answer, "\r\n");
								 send(socket_id, answer, strlen(answer), 0);
								 error = 0;
								}
							  }
							}
						 if (strcmp(request, "SET") == 0)
						   {
							// commando SET
							name = strtok(NULL, ",\r\n\0");
							if (name != NULL)
							  {
							   phone = strtok(NULL, ",\r\n\0");
							   if (phone != NULL)
								 {
								  if (strlen(name)>0 && strlen(phone)>0)
									{
									 if (update(name, phone) > 0)
									   {
										// invio risposta al client
										send(socket_id, "OK\r\n", 4, 0);
										// salvataggio su file
										saveToFile("rubrica.csv");
										error = 0;
									   }
									}
								 }
							  }
							}
						 }
					  }
					if (error) // invio messaggio di errore al client
					  send(socket_id, "ERROR\r\n", 7, 0);
					index = 0;
					break;
				  }
				else
					{
					 // copia del carattere nel comando
					 if (index >= sizeof(command))
					   index = 0;
					 command[index] = buffer[i];
					 index++;
					}
			}
	}
}


int main(void)
{
 int request_socket_id; // socket richieste di connessione
 int communication_socket_id; // socket comunicazione con client
 struct sockaddr_in server_add; // struttura per indirizzo server
 struct sockaddr_in client_add; // struttura per indirizzo client
 size_t client_add_size;
 int n_product;

 shared = shared_memory_alloc(sizeof(struct PHONEBOOK));
 mutex = semaphore_init(1);


 // caricamento prodotti da file
 if ((n_product = loadFromFile("rubrica.csv")) <= 0)
  {
   printf("Errore caricamento prodotti!\r\n");
   shared_memory_free(shared);
   semaphore_remove(mutex);  
   return -1;
 }
 // apertura del socket
 if ( (request_socket_id = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP)) < 0)
  {
   printf("Errore apertura socket!\r\n");
   shared_memory_free(shared);
   semaphore_remove(mutex);  
   return -1;
  }
 // costruzione della struttura per l’indirizzo del server
 memset(&server_add, 0, sizeof(server_add));
 server_add.sin_family = AF_INET;
 server_add.sin_addr.s_addr = 0; // indirizzo IP locale
 server_add.sin_port = htons(23365); // numero di porta server
 // associazione del numero di porta al socket
 if (bind( request_socket_id, (struct sockaddr *)&server_add, sizeof(server_add)) < 0)
   {
    printf("Errore associazione socket!\r\n");
    shared_memory_free(shared);
    semaphore_remove(mutex);  
    close(request_socket_id);
    return -1;
   }
 // impostazione del socket per ricevere le richieste di connessione
 if (listen(request_socket_id, 0) < 0)
   {
    printf("Errore impostazione socket!\r\n");
    close(request_socket_id);
    shared_memory_free(shared);
    semaphore_remove(mutex);  
    return -1;
   }
 printf("Servizio attivo (caricati %i prodotti)…\r\n", n_product);
 while (1)
	  {
	   communication_socket_id = accept(request_socket_id, (struct sockaddr*)&client_add, &client_add_size);
	   if (communication_socket_id >= 0)
		 {
		  // connessione effettuata: creazione nuovo processo per la gestione della comunicazione con il processo client
		  if (fork() == 0)
			{
			 client_service(communication_socket_id);
			 close (communication_socket_id);
			 return 0;
			}
		  close (communication_socket_id);
		 }
	  }
 shared_memory_free(shared);
 semaphore_remove(mutex);  
 close(request_socket_id);
 return 0;
}

